@@ -26,11 +26,10 @@
 
 #include "dispatcher.hh"
 
-Dispatcher::Dispatcher() : err(false)
-{
+Dispatcher::Dispatcher() : err(false) {
+
 }
 
-Dispatcher::~Dispatcher()
-{
+Dispatcher::~Dispatcher() {
 
 }
