@@ -27,10 +27,16 @@
 #include <glibmm/thread.h>
 #include "katoob.hh"
 #include "macros.h"
-#include <csignal>
+//#include <csignal>
 
-int main(int argc, char *argv[])
-{
+/**
+ * \brief Our entry point.
+ *
+ * Our entry point. Will setup gettext, initialize gthread,
+ * construct a Katoob instance, ask Katoob to create a Window and ask Katoob
+ * to run its main loop.
+ */
+int main(int argc, char *argv[]) {
 #ifdef ENABLE_NLS
   bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
   bind_textdomain_codeset (PACKAGE, "UTF-8");
@@ -39,7 +45,10 @@ int main(int argc, char *argv[])
 
   //  signal(SIGPIPE, SIG_IGN);
 
-  if(!Glib::thread_supported()) Glib::thread_init();
+  if(!Glib::thread_supported()) {
+    Glib::thread_init();
+  }
+
   Katoob katoob(argc, argv);
   katoob.window();
   return katoob.run();
