@@ -52,7 +52,8 @@ char *sections[] = {
   _("Print"),
 #endif
   _("Dictionary"),
-  _("Remote"),
+  _("Multipress"),
+  _("Remote Documents"),
   _("Advanced"),
   NULL
 };
@@ -65,12 +66,14 @@ PreferencesDialog::PreferencesDialog(Conf& conf, Encodings& enc) :
   exec_adj(0,0,100),
   undo_closed_adj(0,0,100),
   recentno_adj(0,0,100),
+  tab_width_adj(0,0,10),
 #ifdef ENABLE_PRINT
   dpi_x_adj(0,0,0),
   dpi_y_adj(0,0,0),
 #endif
   dict_port_adj(0,0,0),
   dict_timeout_adj(0,0,0),
+  multipress_timeout_adj(0,0,0),
   locations_size_adj(0,0,0),
   x_adj(0,0,0),
   y_adj(0,0,0),
@@ -100,14 +103,13 @@ PreferencesDialog::PreferencesDialog(Conf& conf, Encodings& enc) :
   selection->signal_changed().connect(sigc::mem_fun(*this, &PreferencesDialog::selection_signal_changed_cb));
   char **tmp = sections;
   int i = 0;
-  while (*tmp)
-    {
-      Gtk::TreeModel::Row row = *(store->append ());
-      row[number] = i;
-      row[section] = _(*tmp);
-      ++tmp;
-      ++i;
-    }
+  while (*tmp) {
+    Gtk::TreeModel::Row row = *(store->append ());
+    row[number] = i;
+    row[section] = _(*tmp);
+    ++tmp;
+    ++i;
+  }
 
   box.pack_start(notebook);
   notebook.set_show_tabs(false);
@@ -124,6 +126,7 @@ PreferencesDialog::PreferencesDialog(Conf& conf, Encodings& enc) :
 #ifdef ENABLE_PRINT
   _print.set_spacing(5);
 #endif
+  _multipress.set_spacing(5);
   _dictionary.set_spacing(5);
   _remote.set_spacing(5);
   _advanced.set_spacing(5);
@@ -140,6 +143,7 @@ PreferencesDialog::PreferencesDialog(Conf& conf, Encodings& enc) :
 #ifdef ENABLE_PRINT
   _print.set_border_width(10);
 #endif
+  _multipress.set_border_width(10);
   _dictionary.set_border_width(10);
   _remote.set_border_width(10);
   _advanced.set_border_width(10);
@@ -159,6 +163,7 @@ PreferencesDialog::PreferencesDialog(Conf& conf, Encodings& enc) :
   print();
 #endif
   dictionary();
+  multipress();
   remote();
   advanced();
 
@@ -170,24 +175,20 @@ PreferencesDialog::PreferencesDialog(Conf& conf, Encodings& enc) :
   apply.signal_clicked().connect(sigc::mem_fun(*this, &PreferencesDialog::apply_clicked_cb));
 }
 
-PreferencesDialog::~PreferencesDialog()
-{
+PreferencesDialog::~PreferencesDialog() {
 
 }
 
-bool PreferencesDialog::run()
-{
+bool PreferencesDialog::run() {
   dialog.show_all();
-  if (dialog.run() == Gtk::RESPONSE_OK)
-    {
-      repopulate_conf();
-      return true;
-    }
+  if (dialog.run() == Gtk::RESPONSE_OK) {
+    repopulate_conf();
+    return true;
+  }
   return false;
 }
 
-void PreferencesDialog::general()
-{
+void PreferencesDialog::general() {
   notebook.append_page(_general);
 
   undo.set_use_underline();
@@ -230,8 +231,7 @@ void PreferencesDialog::general()
   undo_closed.toggled();
 }
 
-void PreferencesDialog::interface()
-{
+void PreferencesDialog::interface() {
   notebook.append_page(_interface);
 
   toolbar.set_use_underline();
@@ -277,14 +277,18 @@ void PreferencesDialog::interface()
   recent.set_active(_conf.get("recent", true));
   showclose.set_active(_conf.get("showclose", true));
   std::string type = _conf.get("toolbartype","both");
-  if (type == "text")
+  if (type == "text") {
     toolbartype.set_active_text(_("Text only"));
-  else if (type == "icons")
+  }
+  else if (type == "icons") {
     toolbartype.set_active_text(_("Icons only"));
-  else if (type == "both_horiz")
+  }
+  else if (type == "both_horiz") {
     toolbartype.set_active_text(_("Text beside icons"));
-  else
+  }
+  else {
     toolbartype.set_active_text(_("Both"));
+  }
 
   toolbar.signal_toggled().connect(sigc::mem_fun(*this, &PreferencesDialog::toolbar_toggled_cb));
   recent.signal_toggled().connect(sigc::mem_fun(*this, &PreferencesDialog::recent_toggled_cb));
@@ -292,8 +296,7 @@ void PreferencesDialog::interface()
   recent.toggled();
 }
 
-void PreferencesDialog::tabs()
-{
+void PreferencesDialog::tabs() {
   notebook.append_page(_tabs);
 
   showtabs.set_use_underline();
@@ -351,28 +354,26 @@ void PreferencesDialog::tabs()
   showtabs.set_active(_conf.get("showtabs", true));
   tabsmenu.set_active(_conf.get("tabsmenu", true));
   scrolltabs.set_active(_conf.get("scrolltabs", true));
-  switch(_conf.get("tabspos", TABS_POS_TOP))
-    {
-    case TABS_POS_BOTTOM:
-      tabspos.set_active_text(_("Bottom"));
-      break;
-    case TABS_POS_RIGHT:
-      tabspos.set_active_text(_("Right"));
-      break;
-    case TABS_POS_LEFT:
-      tabspos.set_active_text(_("Left"));
-      break;
-    default:
-      tabspos.set_active_text(_("Top"));
-      break;
-    }
+  switch(_conf.get("tabspos", TABS_POS_TOP)) {
+  case TABS_POS_BOTTOM:
+    tabspos.set_active_text(_("Bottom"));
+    break;
+  case TABS_POS_RIGHT:
+    tabspos.set_active_text(_("Right"));
+    break;
+  case TABS_POS_LEFT:
+    tabspos.set_active_text(_("Left"));
+    break;
+  default:
+    tabspos.set_active_text(_("Top"));
+    break;
+  }
 
   showtabs.signal_toggled().connect(sigc::mem_fun(*this, &PreferencesDialog::showtabs_toggled_cb));
   showtabs.toggled();
 }
 
-void PreferencesDialog::editor()
-{
+void PreferencesDialog::editor() {
   notebook.append_page(_editor);
 
   textwrap.set_use_underline();
@@ -394,6 +395,9 @@ void PreferencesDialog::editor()
   linenumbers_pos.append_text(_("Both"));
   font_frame.set_label(_("Font"));
 
+  tab_width_label.set_text(_("Tab width"));
+  tab_width.set_adjustment(tab_width_adj);
+
   _editor.pack_start(textwrap, false, false);
   _editor.pack_start(linenumbers, false, false);
   _editor.pack_start(ed_box1, false, false);
@@ -401,12 +405,15 @@ void PreferencesDialog::editor()
 #ifdef ENABLE_HIGHLIGHT
   _editor.pack_start(highlight_auto, false, false);
 #endif
+  _editor.pack_start(ed_box3, false, false);
 
   font_frame.add(ed_box2);
   ed_box2.pack_start(default_font);
   ed_box2.pack_start(font);
   ed_box1.pack_start(linenumbers_label);
   ed_box1.pack_start(linenumbers_pos);
+  ed_box3.pack_start(tab_width_label);
+  ed_box3.pack_start(tab_width);
 
   textwrap.set_active(_conf.get("textwrap", true));
   linenumbers.set_active(_conf.get("linenumbers", false));
@@ -417,27 +424,32 @@ void PreferencesDialog::editor()
 
   bool r = _conf.get("numbers_right", true);
   bool l = _conf.get("numbers_left", true);
-  if ((l) && (r))
+  if ((l) && (r)) {
     linenumbers_pos.set_active_text(_("Both"));
-  else if (l)
+  }
+  else if (l) {
     linenumbers_pos.set_active_text(_("Left"));
-  else if (r)
+  }
+  else if (r) {
     linenumbers_pos.set_active_text(_("Right"));
-  else
+  }
+  else {
     linenumbers_pos.set_active_text(_("Both"));
+  }
 
   std::string _df = Utils::katoob_get_default_font();
   std::string def_font = _conf.get("font", _df);
   font.set_font_name(def_font);
 
+  tab_width_adj.set_value(_conf.get("tab_width", 8));
+
   linenumbers.signal_toggled().connect(sigc::mem_fun(*this, &PreferencesDialog::linenumbers_toggled_cb));
   default_font.signal_toggled().connect(sigc::mem_fun(*this, &PreferencesDialog::default_font_toggled_cb));
   linenumbers.toggled();
   default_font.toggled();
 }
 
-void PreferencesDialog::encodings()
-{
+void PreferencesDialog::encodings() {
   notebook.append_page(_encodings);
 
   locale_enc.set_use_underline();
@@ -463,12 +475,14 @@ void PreferencesDialog::encodings()
   enc_box2.pack_start(save_enc_label, false, false);
   enc_box2.pack_start(save_enc, false, false);
 
-  for (int i = 0; i < _enc.size(); i++)
-    if (i != _enc.utf8())
+  for (int i = 0; i < _enc.size(); i++) {
+    if (i != _enc.utf8()) {
       saved_enc.append_text(_enc.at(i));
-
-  for (int i = 0; i < _enc.size(); i++)
+    }
+  }
+  for (int i = 0; i < _enc.size(); i++) {
     save_enc.append_text(_enc.at(i));
+  }
 
   saved_enc.set_active(_enc.default_open());
   save_enc.set_active(_enc.default_save());
@@ -477,8 +491,7 @@ void PreferencesDialog::encodings()
   locale_enc.toggled();
 }
 
-void PreferencesDialog::file_save()
-{
+void PreferencesDialog::file_save() {
   notebook.append_page(_file_save);
 
   backup.set_use_underline();
@@ -498,8 +511,7 @@ void PreferencesDialog::file_save()
 }
 
 #ifdef HAVE_SPELL
-void PreferencesDialog::spell_checker()
-{
+void PreferencesDialog::spell_checker() {
   notebook.append_page(_spell_checker);
 
   spell_check.set_use_underline();
@@ -511,8 +523,9 @@ void PreferencesDialog::spell_checker()
   std::vector<std::string> dicts;
   katoob_spell_list_available(dicts);
 
-  for (unsigned x = 0; x < dicts.size(); x++)
+  for (unsigned x = 0; x < dicts.size(); x++) {
     default_dict.append_text(dicts[x]);
+  }
 
   _spell_checker.pack_start(spell_check, false, false);
   //  _spell_checker.pack_start(sc_box1, false, false);
@@ -539,8 +552,7 @@ void PreferencesDialog::spell_checker()
 #endif
 
 #ifdef ENABLE_PRINT
-void PreferencesDialog::print()
-{
+void PreferencesDialog::print() {
   notebook.append_page(_print);
 
   print_label.set_text(_("Printing font"));
@@ -586,8 +598,7 @@ void PreferencesDialog::print()
 }
 #endif
 
-void PreferencesDialog::dictionary()
-{
+void PreferencesDialog::dictionary() {
   notebook.append_page(_dictionary);
 
   dict.set_use_underline();
@@ -648,8 +659,20 @@ void PreferencesDialog::dictionary()
   list_dicts.signal_clicked().connect(sigc::mem_fun(*this, &PreferencesDialog::list_dicts_clicked_cb));
 }
 
-void PreferencesDialog::remote()
-{
+void PreferencesDialog::multipress() {
+  notebook.append_page(_multipress);
+  multipress_timeout_l.set_use_underline();
+  multipress_timeout_l.set_label(_("_Milliseconds before multipress will accept the current value."));
+  multipress_timeout.set_adjustment(multipress_timeout_adj);
+  multipress_timeout_adj.set_upper(2000);
+  multipress_timeout_adj.set_lower(200);
+  multipress_timeout_adj.set_value(_conf.get("multipress_timeout", 1000));
+  m_box1.pack_start(multipress_timeout_l);
+  m_box1.pack_start(multipress_timeout);
+  _multipress.pack_start(m_box1, false, false);
+}
+
+void PreferencesDialog::remote() {
   notebook.append_page(_remote);
 
   _remote.pack_start(open_location_to_active, false, false);
@@ -677,8 +700,7 @@ void PreferencesDialog::remote()
   open_location_to_active.set_label(_("_Insert into the active document enabled by default ?"));
 }
 
-void PreferencesDialog::advanced()
-{
+void PreferencesDialog::advanced() {
   notebook.append_page(_advanced);
 
   x_label.set_text(_("Left"));
@@ -737,55 +759,48 @@ void PreferencesDialog::advanced()
   savewinpos.toggled();
 }
 
-void PreferencesDialog::selection_signal_changed_cb()
-{
+void PreferencesDialog::selection_signal_changed_cb() {
   Gtk::TreeModel::iterator iter = selection->get_selected();
-  if (iter)
-    {
-      Gtk::TreeModel::Row row = *iter;
-      int n = row[number];
-      notebook.set_current_page(n);
-    }
+  if (iter) {
+    Gtk::TreeModel::Row row = *iter;
+    int n = row[number];
+    notebook.set_current_page(n);
+  }
 }
 
-void PreferencesDialog::list_dicts_clicked_cb()
-{
+void PreferencesDialog::list_dicts_clicked_cb() {
   std::string host(dict_host.get_text());
   int port = dict_port.get_value_as_int();
   int timeout = dict_timeout.get_value_as_int();
-  if (host.size() > 0)
-    {
-      Dict dict(host, port, timeout);
-      std::string error;
-      if (dict.ok(error))
-	{
-	  Books books;
-	  if (dict.get_books(books, error))
-	    {
-	      dict_store->clear();
-	      Gtk::TreeModel::Row _row;
-	      for (unsigned x = 0; x < books.size(); x++)
-		{
-		  Gtk::TreeModel::Row row = *(dict_store->append());
-		  if (x == 0)
-		    _row = row;
-		  row[dict_name] = books[x].book;
-		  row[dict_description] = books[x].description;
-		}
-	      dict_selection->select(_row);
-	    }
-	  else {
-	    katoob_error(error);
+  if (host.size() > 0) {
+    Dict dict(host, port, timeout);
+    std::string error;
+    if (dict.ok(error)) {
+      Books books;
+      if (dict.get_books(books, error)) {
+	dict_store->clear();
+	Gtk::TreeModel::Row _row;
+	for (unsigned x = 0; x < books.size(); x++) {
+	  Gtk::TreeModel::Row row = *(dict_store->append());
+	  if (x == 0) {
+	    _row = row;
 	  }
+	  row[dict_name] = books[x].book;
+	  row[dict_description] = books[x].description;
 	}
+	dict_selection->select(_row);
+      }
       else {
 	katoob_error(error);
       }
     }
+    else {
+      katoob_error(error);
+    }
+  }
 }
 
-void PreferencesDialog::repopulate_conf()
-{
+void PreferencesDialog::repopulate_conf() {
   _conf.set("undo", undo.get_active());
   _conf.set("undo_closed", undo_closed.get_active());
   _conf.set("undono", undono.get_value_as_int());
@@ -799,23 +814,23 @@ void PreferencesDialog::repopulate_conf()
   _conf.set("showclose", showclose.get_active());
   _conf.set("recentno", recentno.get_value_as_int());
   _conf.set("extra_buttons", extra_buttons.get_active());
+  _conf.set("tab_width", tab_width.get_value_as_int());
 
   std::string val;
-  switch (toolbartype.get_active_row_number())
-    {
-    case 0:
-      val = "text";
-      break;
-    case 1:
-      val = "icons";
-      break;
-    case 3:
-      val = "both_horiz";
-      break;
+  switch (toolbartype.get_active_row_number()) {
+  case 0:
+    val = "text";
+    break;
+  case 1:
+    val = "icons";
+    break;
+  case 3:
+    val = "both_horiz";
+    break;
     default:
       val = "both";
       break;
-    }
+  }
   _conf.set("toolbartype", val.c_str());
 
   _conf.set("showtabs", showtabs.get_active());
@@ -823,21 +838,20 @@ void PreferencesDialog::repopulate_conf()
   _conf.set("scrolltabs", scrolltabs.get_active());
 
   int i;
-  switch (tabspos.get_active_row_number())
-    {
-    case 1:
-      i = TABS_POS_BOTTOM;
+  switch (tabspos.get_active_row_number()) {
+  case 1:
+    i = TABS_POS_BOTTOM;
+    break;
+  case 2:
+    i = TABS_POS_RIGHT;
+    break;
+  case 3:
+    i = TABS_POS_LEFT;
       break;
-    case 2:
-      i = TABS_POS_RIGHT;
-      break;
-    case 3:
-      i = TABS_POS_LEFT;
-      break;
-    default:
-      i = TABS_POS_TOP;
-      break;
-    }
+  default:
+    i = TABS_POS_TOP;
+    break;
+  }
   _conf.set("tabspos", i);
   Gdk::Color color = readonly.get_color();
   _conf.set("readonly_red", color.get_red());
@@ -861,32 +875,32 @@ void PreferencesDialog::repopulate_conf()
 #endif
 
   bool r, l;
-  switch (linenumbers_pos.get_active_row_number())
-    {
-    case 0:
-      l = false;
-      r = true;
-      break;
-    case 1:
-      l = true;
-      r = false;
-      break;
-    default:
-      l = true;
-      r = true;
-      break;
-    }
+  switch (linenumbers_pos.get_active_row_number()) {
+  case 0:
+    l = false;
+    r = true;
+    break;
+  case 1:
+    l = true;
+    r = false;
+    break;
+  default:
+    l = true;
+    r = true;
+    break;
+  }
   _conf.set("numbers_right", r);
   _conf.set("numbers_left", l);
 
-  _conf.set("save_enc", _enc.get(save_enc.get_active_row_number()).c_str());
-  _conf.set("saved_enc", _enc.get(saved_enc.get_active_row_number()).c_str());
+  _conf.set("save_enc", _enc.get_charset(save_enc.get_active_row_number()).c_str());
+  _conf.set("saved_enc", _enc.get_charset(saved_enc.get_active_row_number()).c_str());
   _conf.set("locale_enc", locale_enc.get_active());
   _conf.defaults(_enc);
 
   _conf.set("backup", backup.get_active());
-  if (backup_ext.get_text().size() == 0)
+  if (backup_ext.get_text().size() == 0) {
     backup_ext.set_text("~");
+  }
   _conf.set("backup_ext", backup_ext.get_text().c_str());
 
 #ifdef HAVE_SPELL
@@ -911,12 +925,13 @@ void PreferencesDialog::repopulate_conf()
   _conf.set("dict_timeout", dict_timeout.get_value_as_int());
 
   Gtk::TreeModel::iterator iter = dict_selection->get_selected();
-  if (iter)
-    {
-      Gtk::TreeModel::Row row = *iter;
-      Glib::ustring word = row[dict_name];
-      _conf.set("dict_db", word.c_str());
-    }
+  if (iter) {
+    Gtk::TreeModel::Row row = *iter;
+    Glib::ustring word = row[dict_name];
+    _conf.set("dict_db", word.c_str());
+  }
+
+  _conf.set("multipress_timeout", multipress_timeout.get_value_as_int());
 
   _conf.set("open_location_to_active", open_location_to_active.get_active());
   _conf.set("locations_size", locations_size.get_value_as_int());
