@@ -28,33 +28,39 @@
 
 // TODO: Are we really printing ?
 
-Print::Print(Conf& conf) : PS(conf)
-{
+Print::Print(Conf& conf) : PS(conf) {
 }
 
-Print::~Print()
-{
+Print::~Print() {
 }
 
-void Print::start_process()
-{
+void Print::start_process() {
   init();
   header();
 }
 
-bool Print::end_process(std::string& err)
-{
+bool Print::end_process(std::string& err) {
   footer();
-  if (PS::process(err))
+  if (PS::process(err)) {
     return PDF::process(PS::get_name(), err);
-  else
+  }
+  else {
     return false;
+  }
 }
 
-bool Print::ok(std::string& er)
-{
-  if (PS::ok(er))
-    return PDF::ok(er);
-  else
-    return false;
+bool Print::ok(std::string& er) {
+  return PS::ok(er) ? PDF::ok(er) : false;
+}
+
+void Print::set_type(int t) {
+  _type = t;
+}
+
+void Print::set_file(std::string& f) {
+  _file = f;
+}
+
+void Print::set_copies(int c) {
+  _copies = c;
 }
